package com.obdscope.servlet;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Enumeration;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * A class that implements OBDScope Remote logging server.
 * 
 * Copyright: Public domain.
 * 
 * @author www.obdscope.com
 *
 */
public class RemoteLoggingServlet extends HttpServlet {
    
    private Hashtable clients = new Hashtable();
    
    /**Initialize global variables*/
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
    }

    /**
     * Overriding HttpServlet.doPost(..).
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        // Client sent data.

        if (request == null)
            throw new IOException("Invalid request");
        try {
            
            // Get the client data to String object:
            int length = request.getContentLength();
            byte[] httpInData = new byte[length];
            ServletOutputStream httpOut = response.getOutputStream();
            ServletInputStream httpIn = request.getInputStream();
            BufferedInputStream bufIn = new BufferedInputStream(httpIn);
            bufIn.read(httpInData, 0, length);
            bufIn.close();
            String data = new String(httpInData);
            
            // Peek the data to find out the Client ID:
            StringTokenizer tok = new StringTokenizer(data, ",;");
            String clientId = "";
            if (tok.hasMoreElements()) {
                clientId = tok.nextToken();
            }
            // System.out.println( "data: " + data);

            // Handle the received data:
            if (data.startsWith("OBDScope ")) {
                // New client sent initialization message
                Client newClient = new Client(data);
                if (!clients.containsKey(newClient.getId())) {
                    clients.put(newClient.getId(), newClient);
                    System.out.println("Client added: " + newClient.getId());
                }
            } else if (data.endsWith("BYE")) {
                // BYE message received
                if (clients.containsKey(clientId)) {
                    System.out.println("Client removed: " + clientId);
                    clients.remove(clientId);
                }
            } else {
                // Client sent data
                ((Client) (clients.get(clientId))).update(data);
            }

            // Send OK response to the client
            BufferedOutputStream bufOut = new BufferedOutputStream(httpOut);
            response.setContentType("text/plain");
            bufOut.write(new String("OK").getBytes());
            bufOut.flush();
            bufOut.close();
        } catch (Exception e) {
            // pass it on ...
            throw new IOException("Request failed" + e);
        }
    }
    
    /**
     * Overriding HttpServlet.doGet(..).
     */
    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        // Browser requested data.
        // Send a HTML page containing list of clients and their latest
        // parameter values.

        PrintWriter out = response.getWriter();
        out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\">");
        out.println("<html><head><meta http-equiv=Content-Type content=\"text/html;\">");
        out.println("<title>OBDScope Remote Logging Servlet</title>");
        out.println("</head><body>");
        if (clients.size() == 0) {
            out.println("No clients connected");
        } else {
            Enumeration enr = clients.elements();
            while (enr.hasMoreElements()) {
                out.println(enr.nextElement().toString());
            }
        }
        out.println("</body></html>");
    }

}
